// Package declaration.
// Every class in Java is present in some package which identifies it.
// The filesystem structure (usually; if it doesn't, javac will break in weird but _mostly_ predictable ways)
// matches the packages.
package org.example;

// A public class declaration.
// Every .class file contains 1 public class.
// It might optionally contain non-public classes, 
// which are only accessible from within this class.
// Every _class_ has some _instances_ in memory when it is _instantiated.
// A _class_ is a template which describes a memory layout, 
// and a bunch of code and attributes on said memory layout.
// Every class has _members_. The main member kinds are _field_ and _method_.
// A _field_ is data, and a _method_ is code that acts on the data.
// Instances live in variables or fields (think of them as pointers to the data, 
// and of classes as structs, in C/Rust terms).
// Additionally, there is one special, implicit instance that exists for _every_ class,
// Which is the static instance. The static instance is created when the class is loaded,
// and only contains members marked as static.
// The purpose of static instances is stuff like singletons, etc.
public class App {
    // The entrypoint. The signature 
    // (unique description of the method, including its name and parameter types and order)
    // is defined for the JVM to be the below.
    // The method has to be static (explained above)
    // return nothing/unit (void return type), 
    // And take an array (T[], where T is any type) of Strings called args.
    // The String here is the class java.lang.String, but the Java compiler
    // implicitly imports some classes for the programmer
    // (specifically, all classes in the java.lang package, which also includes System)
    public static void main(String[] args) {
        // This takes the _static field_ of the class java.lang.System called `out`, and invokes the _println_ method on it.
        // out is a field, and it's an instance of java.io.PrintStream, which defines the println method.
        // String literals instantiate a java.lang.String instance from their contents.
        System.out.println("Hello, world!");
    }
}
